@@ -18,12 +18,18 @@
 #include <bonobo/bonobo-private.h>
 #include <bonobo/bonobo-arg.h>
 
+#ifdef G_OS_WIN32
+#include "bonobo-activation/bonobo-activation-private.h"
+#endif
+
 #include <libintl.h>
 #include <string.h>
 
 #include <glib/gmain.h>
 
+#ifdef HAVE_GTHREADS
 GMutex                   *_bonobo_lock;
+#endif
 CORBA_ORB                 __bonobo_orb = CORBA_OBJECT_NIL;
 PortableServer_POA        __bonobo_poa = CORBA_OBJECT_NIL;
 PortableServer_POAManager __bonobo_poa_manager = CORBA_OBJECT_NIL;
@@ -31,6 +37,13 @@ PortableServer_POAManager __bonobo_poa_manager = CORBA_OBJECT_NIL;
 static guint              bonobo_main_loop_level = 0;
 static GSList *           bonobo_main_loops = NULL;
 
+#ifdef G_OS_WIN32
+
+#undef BONOBO_LOCALEDIR
+#define BONOBO_LOCALEDIR _bonobo_activation_win32_get_localedir ()
+
+#endif
+
 /**
  * bonobo_orb:
  *
@@ -85,7 +98,7 @@ bonobo_is_initialized (void)
 }
 
 /**
- * bonobo_shutdown:
+ * bonobo_debug_shutdown:
  * 
  *   This shuts down the ORB and any other bonobo related
  * resources.
@@ -215,7 +228,9 @@ bonobo_init_full (int *argc, char **argv,
 
 	CORBA_exception_free (&ev);
 
+#ifdef HAVE_GTHREADS
 	_bonobo_lock = g_mutex_new ();
+#endif
 
 	bonobo_context_init ();
 
